unit PropSaveTMSGrids;

{<
  PropSaveTMSGrids contains TTMSColGridsPropSave components for re/storing
  properties of TMS ColumnGrid component.

  Updated version compatible with Delphi2010+.
  @author(Rosinsky Software - <http://www.rosinsky.cz/delphi>)
  @created(Oct 2012)
  @lastmod(Dec 2012)
  }

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  PropSaveMain, AdvCGrid;

type
  TColumnIdentification = (cidTag, cidHeader);

  { descendant of TCustomPropSave component and
    provides re/storing of TAdvColumnGrid properties }
  TTMSColGridsPropSave = class(TCustomPropSave)
  private
    // FStoreColumnsProp: boolean;
    FColumnIdentification: TColumnIdentification;
    procedure SaveTMSColGrid(PSFile: TPropSaveFile; Key: string; SG: TAdvColumnGrid);
    procedure ReadTMSColGrid(PSFile: TPropSaveFile; Key: string; SG: TAdvColumnGrid);
  protected
    procedure ReadSaveProps(A: TPropSaveAction; PSFile: TPropSaveFile; Key: string; Comp: TComponent); override;
  public
    { Return whether re/storing of Components from parameter
      is supported by TTMSColGridsPropSave }
    function CanSave(Comp: TComponent): boolean; override;
  published
    { Define whether columns properties (fonts, color, alignmetn...) is re/stored }
    // property StoreColumnsProp: boolean  read FStoreColumnsProp write FStoreColumnsProp;
    property ColumnIdentification: TColumnIdentification read FColumnIdentification write FColumnIdentification;

  end;

procedure Register;

implementation

{$R *.res}
uses rtool;

const
  sColCount = 'ColCount';
  sRowCount = 'RowCount';
  sRowHeight = 'RowHeight%d';
  sColumn = 'Column%d';
  sWidth = 'Width';

  sColHeader = 'ColHeader';
  sColTag = 'ColTag';
  sColWidth = 'ColWidth';
  sColVisible = 'ColVisible';

procedure Register;
begin
  RegisterComponents('Rosi PS', [TTMSColGridsPropSave]);
end;

// ==============================================================================
// DBGridsPropSave functions
function TTMSColGridsPropSave.CanSave(Comp: TComponent): boolean;
begin
  Result:=(Comp is TAdvColumnGrid);
end;

procedure TTMSColGridsPropSave.ReadSaveProps(A: TPropSaveAction; PSFile: TPropSaveFile; Key: string; Comp: TComponent);
begin
  if (Comp is TAdvColumnGrid) then
  begin
    if A = psaRead then
      ReadTMSColGrid(PSFile, Key, Comp as TAdvColumnGrid)
    else
      SaveTMSColGrid(PSFile, Key, Comp as TAdvColumnGrid);
  end;
end;

procedure TTMSColGridsPropSave.SaveTMSColGrid(PSFile: TPropSaveFile; Key: string; SG: TAdvColumnGrid);
var
  ColIdx: integer;
  k: string;
  c: TGridColumnItem;
begin
  PSFile.WriteInteger(Key, sColCount, SG.Columns.Count);
  for ColIdx:=0 to SG.Columns.Count - 1 do
  begin
    c:=SG.Columns[ColIdx];
    k:=Key + '.' + Format(sColumn, [ColIdx]);
    // Save column position
    case FColumnIdentification of
      cidTag:
        PSFile.WriteInteger(k, sColTag, c.Tag);
      cidHeader:
        PSFile.WriteString(k, sColHeader, c.Header);
    end;
    if c.Width > 0 then
      PSFile.WriteInteger(k, sColWidth, c.Width);
    PSFile.WriteBool(k, sColVisible, c.Width > 0);

    { // Save column props
      if FStoreColumnsProp then
      begin
      if (cvColor in C.AssignedValues) and
      (C.Color <> C.DefaultColor) then PSFile.WriteInteger(k,sColColor,c.Color)
      else PSFile.DeleteKey(k,sColColor);
      if (cvFont in C.AssignedValues) then PSFile.WriteFont(k,c.Font)
      else PSFile.DeleteFont(k);
      if (cvAlignment in C.AssignedValues) then PSFile.WriteInteger(k,sColAlign,Ord(c.Alignment))
      else PSFIle.DeleteKey(k,sColAlign);

      if (cvTitleColor in C.AssignedValues) and
      (C.Title.Color <> C.Title.DefaultColor) then PSFile.WriteInteger(k,sColTitleColor,c.Title.Color)
      else PSFile.DeleteKey(k,sColTitleColor);
      // if (cvTitleFont in C.AssignedValues) then PSFile.WriteFont(k,c.Title.Font);
      if (cvTitleAlignment in C.AssignedValues) then PSFile.WriteInteger(k,sColTitleAlign,Ord(c.Title.Alignment))
      else PSFile.DeleteKey(k,sColTitleAlign);

      if (cvTitleCaption in C.AssignedValues) then PSFile.WriteString(k,sColTitle,c.Title.Caption)
      else PSFile.DeleteKey(k,sColTitle);

      end;
    }
  end;
end;

procedure TTMSColGridsPropSave.ReadTMSColGrid(PSFile: TPropSaveFile; Key: string; SG: TAdvColumnGrid);
var
  SavedCols, GridCols, scIdx, gcIdx, actIdx: integer;
  k, fn: string;
  c: TGridColumnItem;
  ChangedColsIdx: array of integer;
  SavedColFields: array of string;

  function GetColID(c: TGridColumnItem): string;
  begin
    Result:='';
    case FColumnIdentification of
      cidTag:
        Result:=IntToStr(c.Tag);
      cidHeader:
        Result:=c.Header;
    end;
  end;

  function FieldSavedIdx(ColID: string): integer;
  var
    i: integer;
  begin
    Result:=-1;
    for i:=0 to High(SavedColFields) do
      if SameText(ColID, SavedColFields[i]) then
        Result:=i;
  end;

  function MoveIdx(newIdx: integer): integer;
  var
    i: integer;
  begin
    Result:=0;
    if newIdx > High(ChangedColsIdx) then
      newIdx:= High(ChangedColsIdx);
    for i:=0 to newIdx - 1 do
      Inc(Result, ChangedColsIdx[i]);
    for i:=newIdx to High(ChangedColsIdx) do
      if ChangedColsIdx[i] = 1 then
        Inc(Result)
      else
        Break;
  end;

begin
  SavedCols:=PSFile.ReadInteger(Key, sColCount, -1);
  GridCols:=SG.Columns.Count;

  if SavedCols > 0 then
    SetLength(SavedColFields, SavedCols);
  for scIdx:=0 to SavedCols - 1 do
  begin
    // Read column by index
    k:=Key + '.' + Format(sColumn, [scIdx]);
    case FColumnIdentification of
      cidTag:
        SavedColFields[scIdx]:=PSFile.ReadString(k, sColTag, '');
      cidHeader:
        SavedColFields[scIdx]:=PSFile.ReadString(k, sColHeader, '');
    end;
  end;

  SetLength(ChangedColsIdx, GridCols);
  for gcIdx:=0 to GridCols - 1 do
  begin
    if FieldSavedIdx(GetColID(SG.Columns[gcIdx])) < 0 then
      ChangedColsIdx[gcIdx]:=1
    else
      ChangedColsIdx[gcIdx]:=0;
  end;

  for scIdx:=0 to SavedCols - 1 do
  begin
    // Read column by index
    k:=Key + '.' + Format(sColumn, [scIdx]);
    case FColumnIdentification of
      cidTag:
        fn:=PSFile.ReadString(k, sColTag, '');
      cidHeader:
        fn:=PSFile.ReadString(k, sColHeader, '');
    end;
    // find same column in grid (according tag)
    actIdx:=-1;
    for gcIdx:=0 to SG.Columns.Count - 1 do
      if GetColID(SG.Columns[gcIdx]) = fn then
      begin
        actIdx:=gcIdx;
        Break;
      end;
    if actIdx < 0 then
    begin
      PSFile.EraseSection(k);
      Continue;
    end;

    c:=SG.Columns[actIdx];
    actIdx:=scIdx + MoveIdx(scIdx);
    if actIdx >= GridCols then
      actIdx:=GridCols - 1;
    c.Index:=actIdx;
    // if not inList then
    c.Width:=PSFile.ReadInteger(k, sColWidth, c.Width);
    if not PSFile.ReadBool(k, sColVisible, c.Width > 0) then
      c.Width:=0;

    { // Read column props
      if FStoreColumnsProp then
      begin
      SavedInt:=PSFile.ReadInteger(k,sColColor,-1);
      if SavedInt<>-1 then c.Color:=SavedInt;
      PSFile.ReadFont(k,c.Font);
      SavedInt:=PSFile.ReadInteger(k,sColAlign,-1);
      if SavedInt<>-1 then try c.Alignment:=TAlignment(SavedInt); except end;

      SavedInt:=PSFile.ReadInteger(k,sColTitleColor,-1);
      if SavedInt<>-1 then c.Title.Color:=SavedInt;
      //  PSFile.ReadFont(k,c.Title.Font);
      SavedInt:=PSFile.ReadInteger(k,sColTitleAlign,-1);
      if SavedInt<>-1 then try c.Title.Alignment:=TAlignment(SavedInt); except end;

      SavedStr:=PSFile.ReadString(k,sColTitle,'');
      if SavedStr<>'' then c.Title.Caption:=SavedStr;
      end;
    }
    if scIdx >= SG.Columns.Count then
      PSFile.EraseSection(k);
  end;
end;

end.
