unit PropSave_DTEdit;

{<
  PropSave_DTEdit is design time dialog used by PropSave components.

  Updated version compatible with Delphi2010+.
  Full functional demo.
  @author(Rosinsky Software - <http://www.rosinsky.cz/delphi>)
  @created(May 2006)
  @lastmod(Jan 2013)
}

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, Buttons, ComCtrls, PropSaveMain, rImprovedComps, rHTMLLabel;

type
  { design time dialog used by PropSaveMain component }
  TPropSave_DTEditForm = class(TForm)
    tc: TTabControl;
    gbEmpty: TrGroupBox;
    btClose: TButton;
    gbLeft: TrGroupBox;
    Label1: TLabel;
    lbComps: TListBox;
    gbRight: TrGroupBox;
    Label2: TLabel;
    lbSave: TListBox;
    btMove1: TButton;
    btMoveAll: TButton;
    btDel1: TButton;
    btDelAll: TButton;
    btSort: TrURLLabel;
    btDelOrphan: TButton;
    PSFile: TPropSaveFile;
    PSMain: TPropSaveMain;
    procedure FormShow(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure btCloseClick(Sender: TObject);
    procedure tcChange(Sender: TObject);
    procedure tcChanging(Sender: TObject; var AllowChange: Boolean);
    procedure btMove1Click(Sender: TObject);
    procedure btMoveAllClick(Sender: TObject);
    procedure btDel1Click(Sender: TObject);
    procedure btDelAllClick(Sender: TObject);
    procedure FormResize(Sender: TObject);
    procedure btSortClick(Sender: TObject);
    procedure lbSaveDragOver(Sender, Source: TObject; X, Y: Integer; State: TDragState; var Accept: Boolean);
    procedure btDelOrphanClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
  private
    Form: TForm;
    List: array [0 .. 1] of TStrings;
    CPS: array [0 .. 1] of TCustomPropSave;
    procedure LoadProps;
    procedure SetProps;
    procedure ShowControls;
  public
  end;

function ShowPropertySaverEditor(Component: TComponent): Boolean;

implementation

{$R *.dfm}
uses TypInfo, RosiCompGlobalCfg, rtool, rdlg;

const
  cTag = 'Tag';

function ShowPropertySaverEditor(Component: TComponent): Boolean;
var
  F: TPropSave_DTEditForm;
begin
  Result:=true;
  F:=TPropSave_DTEditForm.Create(Application);
  try
    F.Caption:=Component.Name;
    F.Form:=Component.Owner as TForm;
    F.tc.Tabs.Clear;
    if Component is TPropSaveMain then
    begin
      F.tc.Tabs.Add('StdCtrl');
      F.CPS[0]:=GetObjectProp(Component, 'StdCtrl') as TCustomPropSave;
      F.List[0]:=F.CPS[0].Components as TStrings;
      F.tc.Tabs.Add('CtrlPosition');
      F.CPS[1]:=GetObjectProp(Component, 'CtrlPosition') as TCustomPropSave;
      F.List[1]:=F.CPS[1].Components as TStrings;
    end
    else
      if Component is TCustomPropSave then
      begin
        F.tc.Tabs.Add('Components');
        F.CPS[0]:=Component as TCustomPropSave;
        F.List[0]:=GetObjectProp(Component, 'Components') as TStrings;
      end
      else
        raise Exception.Create('Unknown component type.');
    F.ShowModal;
  finally
    F.Free;
  end;
end;

procedure TPropSave_DTEditForm.FormCreate(Sender: TObject);
begin
  PSFile.LoadFile(GetRosiCompIniFile);
end;

procedure TPropSave_DTEditForm.FormShow(Sender: TObject);
begin
  tc.TabIndex:=0;
  lbSave.Items.Assign(List[0]);
  LoadProps;
end;

procedure TPropSave_DTEditForm.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  SetProps;
end;

procedure TPropSave_DTEditForm.FormResize(Sender: TObject);
begin
  gbLeft.Width:=tc.ClientWidth div 2 + 20;
end;

procedure TPropSave_DTEditForm.LoadProps;
begin
  ShowControls;
end;

procedure TPropSave_DTEditForm.btSortClick(Sender: TObject);
begin
  lbSave.Sorted:=true;
  lbSave.Sorted:=false;
end;

procedure TPropSave_DTEditForm.btDelOrphanClick(Sender: TObject);
var
  i: Integer;
begin
  for i:=lbSave.Items.Count - 1 downto 0 do
    if lbComps.Items.IndexOf(lbSave.Items[i]) < 0 then
    begin
      lbSave.Items.Delete(i);
    end;
end;

procedure TPropSave_DTEditForm.SetProps;
begin
  List[tc.TabIndex].Assign(lbSave.Items);
end;

procedure TPropSave_DTEditForm.btCloseClick(Sender: TObject);
begin
  Close;
end;

procedure TPropSave_DTEditForm.ShowControls;
  procedure AddComp(C: TComponent; Prefix: string);
  var
    X: Integer;
  begin
    for X:=0 to C.ComponentCount - 1 do
    begin
      if CPS[tc.TabIndex].CanSave(C.Components[X]) then
        if C.Components[X].Name <> '' then
          lbComps.Items.Add(Prefix + C.Components[X].Name);
      AddComp(C.Components[X], Prefix + C.Components[X].Name + '.');
    end;
  end;

begin
  lbComps.Items.Clear;
  AddComp(Form, '');
  lbComps.Sorted:=true;
end;

procedure TPropSave_DTEditForm.tcChanging(Sender: TObject; var AllowChange: Boolean);
begin
  SetProps;
end;

procedure TPropSave_DTEditForm.tcChange(Sender: TObject);
begin
  lbSave.Items.Assign(List[tc.TabIndex]);
  ShowControls;
end;

procedure TPropSave_DTEditForm.btMove1Click(Sender: TObject);
var
  s: string;
begin
  s:=lbComps.Items[lbComps.ItemIndex];
  if lbSave.Items.IndexOf(s) = -1 then
    lbSave.Items.Add(s);
end;

procedure TPropSave_DTEditForm.btMoveAllClick(Sender: TObject);
begin
  lbSave.Items.Assign(lbComps.Items);
end;

procedure TPropSave_DTEditForm.btDel1Click(Sender: TObject);
begin
  lbSave.DeleteSelected;
end;

procedure TPropSave_DTEditForm.btDelAllClick(Sender: TObject);
begin
  lbSave.Items.Clear;
end;

procedure TPropSave_DTEditForm.lbSaveDragOver(Sender, Source: TObject; X, Y: Integer; State: TDragState; var Accept: Boolean);
begin
  Accept:=true;
  MoveListBox(lbSave, lbSave.ItemIndex, Y div lbSave.ItemHeight + lbSave.TopIndex);
end;

end.
